#!/usr/bin/php4 -q
<?php
/*
GOpdc: populate.php4 - Create initial LDAP-tree for later use with GOpdc 

Authors: Jan Wenzel    <jan.wenzel@GONICUS.de>

Copyright (C) 2005 GONICUS GmbH

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Contact information: GONICUS GmbH
Moehnestrasse 11-17
D-59755 Arnsberg
Germany
tel: ++49 2932 916 0
fax: ++49 2932 916 230
email: info@GONICUS.de
http://www.GONICUS.de
*/

# Warning! This script is not working properly! You should only
# test it in an empty LDAP-Directory!
$test_mode= false;

$svn_path= '$HeadURL: https://oss.gonicus.de/repositories/gopdc/trun/usr/share/gopdc/populate.php4 $';
$svn_revision= '$Revision: 109$';

// Read configuration
require_once("/usr/share/gopdc/config.inc");

// Set error_reporting (need to do some checks)
if(defined(getOption('error_reporting'))) {
  // PHP has some defined error constants, like 'E_ALL'
  error_reporting(constant(getOption('error_reporting')));
} else {
  // In this way, "normal" ways of incremental error level can be used: 0-2047
  error_reporting(getOption('error_reporting'));
}

// Map Options from XML-File to local var
/** The primary domain for which GOpdc should alter entries */
$smb_domain=       getOption('smb_domain');

/** "Starting Point" for user id */
$ux_uid=           getOption('ux_uid');

/** "Starting Point" for group id */
$ux_gid=           getOption('ux_gid');

/** The users home directory */
$ux_homedir=       getOption('ux_homedir');

/** 
 * Adds a group to every user and sets its primary group to this group. 
 * Beware that this will break "usrmgr"!
 */
$add_prim_group=   getOption('add_prim_group');

/**
 * Adds Samba Attributes to users primary group.
 * This is only valid with $add_prim_group=true
 */
$add_prim_smb=     getOption('add_prim_smb');

/**
 * Adds new users to group "Domain Users".
 * Only valid if add_prim_group=true
 */
$user_prim_domain= getOption('user_prim_domain');

/** Adds Samba attributes to every new object. This will break "usrmgr" */
$add_samba_attr=   getOption('add_samba_attr');

/** 
 * Verify that RID is not already used. Makes really only sense if 
 * add_samba_attr=true and/or add_prim_smb=true. 
 * You may enable this if you vampired (net rpc vampire) 
 * your domain and have not enough IDs left.
 */
$verify_rid=       getOption('verify_rid');

/** Adds sambaIdMapEntry ObjectClass to samba Accounts. Same prerequisites as $verify_rid */
$add_id_map=       getOption('add_id_map');

/** Hostname of the LDAP-Server */
$ldap_host=        getOption('ldap_host');

/** Base of the LDAP-Tree */
$ldap_base=        getOption('ldap_base');

/** BindDN to use for connection */
$ldap_binddn=      getOption('ldap_binddn');

/** Related password for BindDN */
$ldap_pass=        getOption('ldap_pass');

/** Whether to use TLS to connect to LDAP Server */
$ldap_tls=         getOption('ldap_tls');

/** The port to connect to */
$ldap_port=        getOption('ldap_port');

/** LDAP protocol version to use (Valid values are 1, 2, 3) */
$ldap_prot=        getOption('ldap_prot');

/** Suffix for users. Will be prepended to base, presets to base if not set */
$ldap_suf_user=    getOption('ldap_suf_user');

/** Suffix for groups. Will be prepended to base, presets to base if not set */
$ldap_suf_group=   getOption('ldap_suf_group');

/** Suffix for machines. Will be prepended to base, presets to base if not set */
$ldap_suf_mach=    getOption('ldap_suf_mach');

/** Should a POSIX Account be created */
$create_users=     getOption('create_users');

/** Should the POSIX Account be deleted */
$delete_users=     getOption('delete_users');

/** Should a machine Account be created */
$create_machines=  getOption('create_machines');

/** Should a POSIX Group be created */
$create_groups=    getOption('create_groups');

/** Should the POSIX Group be deleted */
$delete_groups=    getOption('delete_groups');

/** */
$mod_groups=       getOption('mod_groups');

/** Command to be executed after user has been created */
$post_exec_user=   getOption('post_exec_user');

/** Command to be executed after machine account has been created */
$post_exec_mach=   getOption('post_exec_mach');

/** Command to be execeuted after group has been created */
$post_exec_grup=   getOption('post_exec_grup');

/** Program Path to be included */
$prog_path=        getOption('prog_path');

/** Set log channel */
$log_channel=      getOption('log_channel');

// Read Functions
require_once($prog_path."functions.php4");

// Connect to LDAP-Server (this is global)
$ldap_con = ldapConnect();

// checks for valid base entry
function checkBase(){
  global $ldap_base,$ldap_con;

  if ($ldap_base == "") {
    deBug("No LDAP-Base was set. Exiting");
    exit(1);
  } else {
    ldapSearch($ldap_con,"(objectClass=*)",getBase());
  	if( ldap_errno($ldap_con) == 0x20 ||  # LDAP_NO_SUCH_OBJECT
        ldap_errno($ldap_con) == 0x40) {  # LDAP_NAMING_VIOLATION
		  deBug("Empty Base. Will try to add base entry");
      return false;
    }
  } 
  return true;
}

/**
 * Simple function to check if needed OU-base entries are present in LDAP.
 * If not, tries to add these.
 */
function checkOU(){
	global $ldap_base,$ldap_suf_user,$ldap_suf_group,$ldap_suf_mach,$ldap_con,$test_mode;
	
	// Check Variables
	if ($ldap_base == "") {
		deBug("No LDAP-Base was set. Exiting");
		exit(1);
	} else if ( 
		$ldap_suf_user == "" || 
		$ldap_suf_group == "" ||
		$ldap_suf_mach == "") {
			deBug("One of the suffixes is empty. Exiting");
			exit(1);
	}

	// Check for empty LDAP (no base)
	ldapSearch($ldap_con,"(objectClass=*)",getBase());
	if(!checkBase()){
		if(writeBaseEntry()) {
			deBug("Base entry added.");
		} else {
			deBug("Failed to add base entry. Please fix this manually.");
			deBug("Error was \"".ldap_err2str(ldap_errno($ldap_con))."\"");
		}
	}

	// Check for user subtree
	ldapSearch($ldap_con,"(objectClass=*)",getBase($ldap_suf_user));
	if( ldap_errno($ldap_con) == 0x20) { # LDAP_NO_SUCH_OBJECT
		deBug("No User OU found. Will try to fix this.");
		if(writeOU($ldap_suf_user)) {
			deBug("User OU entry added.");
		} else {
			deBug("Failed to add User OU entry. Please fix this manually.");
			deBug("Error was \"".ldap_err2str(ldap_errno($ldap_con))."\"");
		}
	}

	// Check for group subtree
	ldapSearch($ldap_con,"(objectClass=*)",getBase($ldap_suf_group));
	if( ldap_errno($ldap_con) == 0x20) { # LDAP_NO_SUCH_OBJECT
		deBug("No Group OU found. Will try to fix this.");
		if(writeOU($ldap_suf_group)) {
			deBug("Group OU entry added.");
		} else {
			deBug("Failed to add Group OU entry. Please fix this manually.");
			deBug("Error was \"".ldap_err2str(ldap_errno($ldap_con))."\"");
		}
	}
	// Check for machine subtree
	ldapSearch($ldap_con,"(objectClass=*)",getBase($ldap_suf_mach));
	if( ldap_errno($ldap_con) == 0x20) { # LDAP_NO_SUCH_OBJECT
		deBug("No Machine OU found. Will try to fix this.");
		if(writeOU($ldap_suf_mach)) {
			deBug("Machine OU entry added.");
		} else {
			deBug("Failed to add Machine OU entry. Please fix this manually.");
			deBug("Error was \"".ldap_err2str(ldap_errno($ldap_con))."\"");
		}
	}
}

/** Adds Domain SID to LDAP-Tree
 * @param $ldap_con Valid LDAP connection handle
 * @return true if success, false otherwise
 */
function addDomainSid(){
 	global $GDATA,$ldap_base,$smb_domain,$ldap_con,$test_mode;
	// Check for empty LDAP (no base)
	checkOU();

  $result= false;

  if(!getDomainSid($ldap_con)) {
    // First try to get the SID via the 'net' command
    if(isset($smb_domain)) {
      $net_cmd = trim(`which net`);
      if($net_cmd == "") {
        deBug("the 'net' commando was not found in path. please fix");
        exit(1);
      }
      
      $cmd_line = "$net_cmd rpc info -w $smb_domain";
      
      // Convert Multi-Line string to one line
      $string = preg_replace("[\n]", " ", `$cmd_line`);
      
      // TODO: Verify SID-Format
      $regex = "/S\-\d\-\d\-\d{2}\-\d{8,10}\-\d{8,10}\-\d{8,10}/";	
      
      // Save SID in $GDATA["domsid"]
      $sid=array();
      preg_match($regex, $string, $sid);
      if(isset($sid[0])){
        $GDATA["domsid"] = $sid[0];
        deBug("Will try to add SID: ".$GDATA["domsid"]);
      }

      if(isset($GDATA["domsid"])) {
        $ldapadd["objectclass"][0]="sambaDomain";
        $ldapadd["sambaDomainName"]=$smb_domain;
        $ldapadd["sambaSID"]=$GDATA["domsid"];

        // RidBase is a "May" for sambaDomain
        if(isset($GDATA["ridbase"])) {
          $ldapadd["sambaAlgorithmicRidBase"]=$GDATA["ridbase"];
        }
        if(!$test_mode){
          if(ldap_add($ldap_con,"sambaDomainName=".$smb_domain.",".$ldap_base,$ldapadd)) {
            if(getDomainSid($ldap_con) == $sid[0]) {
              deBug("Successfully wrote sambaDomain entry");
            }
          } else {
            deBug("Could not insert sambaDomain entry");
          }
        } else {
          print_r($ldapadd);
        }
      } else {
        deBug("Could not insert base entry");
      }
    }
  } else {
    deBug("sambaDomain entry already exists in your LDAP");
  }
  
  return $result;
}

/**
 * Will try to write ou for given suffix
 * @param $organizationalUnit suffix for ou
 * @return true if added, false otherwise
 */
function writeOU($organizationalUnit) {
	global $ldap_base,$ldap_con,$test_mode;
	$local_base = $ldap_base;
  $result="";
	// Check if we need to add multiple entries (as for ou=people,ou=Machines)
	$entries = preg_split("/,/", $organizationalUnit);
	foreach(array_reverse($entries) as $entry) {
		$ou = array_pop(preg_split("/=/", $entry));
		$ldapadd["objectclass"]="organizationalUnit";
		$ldapadd["ou"]=$ou;
    if(!$test_mode) {
		  $result = ldap_add($ldap_con,$entry.",".$local_base,$ldapadd);
		  if(!$result){
			  print("Function writeOU: Error was ".ldap_error($ldap_con)."\n");
		  }
		  $local_base = "ou=".$ou.",".$local_base;
    } else {
      print("Function writeOU: I would write the following:\n");
      print_r($ldapadd);
    }
	}
	return $result;
}

// Writes top Entry for empty LDAP tree
function writeBaseEntry(){
	global $ldap_base,$ldap_con,$test_mode;
	$organization = array_pop(preg_split("/=/", array_shift(preg_split("/,/", $ldap_base))));
  $ldapadd["objectclass"][0]="top";
	$ldapadd["objectclass"][1]="organization";
  $ldapadd["objectclass"][2]="dcObject";
	$ldapadd["dc"]=$organization;
  $ldapadd["o"]=$organization;
  if(!$test_mode){
    return(ldap_add($ldap_con,$ldap_base,$ldapadd));
  } else {
    print("Function writeBaseEntry: I would write the following:\n");
    print_r($ldapadd);
    return 0;
  }
}

function checkStandardGroups() {
	global $ldap_base,$ldap_con,$ldap_suf_group,$ldap_suf_mach,$GDATA,$test_mode;
	$group_base = getBase($ldap_suf_group);
	$mach_base = getBase($ldap_suf_mach);

	// First try to find the user entries
	$result = getPrimaryGroup($ldap_con,"users");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-513";
		$ldapadd['cn']="domain users";
		
		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
		if(!$test_mode){
      ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
    } else {
    print("Function checkStandardGroups: I would write the following:\n");
      print_r($ldapadd);
    }
	}

	// admins
	$result = getPrimaryGroup($ldap_con,"admins");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-512";
		$ldapadd['cn']="domain admins";

		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
    if(!$test_mode){
    	ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
    } else {
      print_r($ldapadd);
    }
	}
	 // guests
	$result = getPrimaryGroup($ldap_con,"guests");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-514";
		$ldapadd['cn']="domain guests";
		
		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
    if(!$test_mode){
    	ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
    } else {
      print_r($ldapadd);
    }
	}

	// machines
	$result = getPrimaryGroup($ldap_con,"machines");
	if(!$result) {
		$ldapadd['objectClass'][0]="sambaGroupMapping";
		$ldapadd['objectClass'][1]="posixGroup";
		$ldapadd['sambaGroupType']="2";
		$ldapadd['sambaSID']=getDomainSid($ldap_con)."-516";
		$ldapadd['cn']="domain machines";
		
		// Don't allow GID-caching
		unset($GDATA['gid']);
		$ldapadd['gidNumber']=getNextID($ldap_con,"gid");
    if(!$test_mode){
    	ldap_add($ldap_con,"cn=".$ldapadd['cn'].",".$group_base,$ldapadd);
    } else {
      print_r($ldapadd);
    }
	}
}
	
function populate() {
  if(!checkBase()) {
  	// First try to write the base entry
	  writeBaseEntry();
    
    // Call script again
    populate();
  } else {
    // deBug("Will try to write the sambaSID");
	  // Write Domain SID
	  // addDomainSid();
	
   	// Write OUs
	  checkOU();
  }
}

function test() {
  checkOU();

}

// Parse commandline
if ($argc<2) { help(); }
for($i=1;$i<$argc;$i++){
	switch($argv[$i]) {
		/** @cond */
		case "--help":
			help();
			exit;
			break;
			
		case "--populate":
			populate();
			break;
		
		case "--checkou":
			checkOU();
			break;

		case "--checkgroups":
			checkStandardGroups();
			break;
			
		case "--checkbase":
			checkBase();
			break;
			
		case "--addsid":
			addDomainSid();
			break;

    case "--test":
      test();
      break;

		default:
			help();
			exit(1);
		/** @endcond */
	}
}

function help() {
		$call= "populate.php4";
	
    $help="usage: populate.php4 function ...\n"
    ."  --help\t\tThe text you are reading.\n"
    ."  --populate\t\tCreates required entries in empty LDAP-Tree.
        \t\tThis means top entry for organization and
        \t\tpeople, groups and systems container taken from config.\n"
    ."  --checkbase\t\tChecks for valid Base Entry.\n"
    ."  --checkou\t\tChecks for required OU-Entries for saving user,
        \t\tgroup and machine accounts and creates them.\n"
    ."  --checkgroups\t\tChecks for required Windows Group Mappings.\n"
    ."  --addsid\t\tAdds Domain SID entry to LDAP.
        \t\tYou should better use samba directly.\n"
    ."  --test\t\tDoes all testing.\n"
    ."\nFor a full samba compatible setup, you should call in order:
        1. populate
        2. addsid or start samba once (safe)
        3. checkgroups\n\n";

		print $help;
}

// vim:tabstop=2:expandtab:shiftwidth=2:filetype=php:syntax:ruler:
?>
